package comaList ;


/**
 * Klasse fr doppelt verkettete Listen
 */
public class DoubleList
{
   private class ListNode                       // quasi struct
   {
   /***  Daten  ***/

      public Object   obj ;
      public ListNode next ;
      public ListNode prev ;

   /***  Konstruktoren  ***/

      public ListNode ( )
      {
         obj  = null ;
	 next = prev = null ;
      }

      public ListNode ( Object myObj )
      {
         obj  = myObj ;
 	 next = prev = null ;
      }
   }  // class ListNode


/***  Daten  ***/

   private ListNode _head ;                     // Dummyknoten am Kopf
   private ListNode _tail ;                     // Dummyknoten am Ende
   private ListNode _curr ;                     // akt. Knoten


/***  Konstruktoren  ***/

   public DoubleList ( )
   {
      _head = new ListNode() ;                             // Dummies erzeugen
      _tail = new ListNode() ;
      _curr = null ;

      _head.next = _tail.next = _tail ;                    // leere Liste init.
      _head.prev = _tail.prev = _head ;

   }  // end DoubleList()


   public DoubleList ( DoubleList myList )
   {
      this() ;                                            // leere Liste init.

      for (   myList.resetToHead() ;                      // myList strukturell
            ! myList.isAtTail() ;                         // kopieren
	      myList.increment() )
	 insertAtTail( myList.currentData() ) ;

   }  // end DoubleList()


/***  get-Methoden  ***/

   public boolean isEmpty ( )
   {
      return ( _head.next == _tail ) ;
   }

   public boolean isAtHead ( )
   {
      return ( _curr == _head ) ;
   }

   public boolean isAtTail ( )
   {
      return ( _curr == _tail ) ;
   }

   public Object currentData ( )
   {
      return _curr.obj ;
   }

   public int length ( )
   {
      int len = 0 ;

      for ( resetToHead() ;  ! isAtTail() ;  increment() )
	 ++len ;

      return len ;

   }  // end length()


/***  set-Methoden  ***/

   public void resetToHead ( )
   {
      _curr = _head.next ;
   }

   public void resetToTail ( )
   {
      _curr = _tail.prev ;
   }

   public void increment ( )
   {
      _curr = _curr.next ;
   }

   public void decrement ( )
   {
      _curr = _curr.prev ;
   }


   public void insertAtHead ( Object obj )
   {
      ListNode node = new ListNode( obj ) ;

      node.next  = _head.next ;
      node.prev  = _head ;
      _head.next = _head.next.prev = node ;

   }  // end insertAtHead()


   public void insertAtTail ( Object obj )
   {
      ListNode node = new ListNode( obj ) ;

      node.next  = _tail ;
      node.prev  = _tail.prev ;
      _tail.prev = _tail.prev.next = node ;

   }  // end insertAtTail()


   private void _delete ( ListNode node )
   {
      node.next.prev = node.prev ;
      node.prev.next = node.next ;
   }

    
   public void remove ( Object obj )
   {
      for ( resetToHead() ;  ! isAtTail() ;  increment() )
	 if ( obj.equals( _curr.obj ) )
	    _delete( _curr ) ;
   }


   public void removeAtHead ( )
   {
      if ( ! isEmpty() )
	 _delete( _head.next ) ;
   }


   public void removeAtTail ( )
   {
      if ( ! isEmpty() )
	 _delete( _tail.prev ) ;
   }


/***  Umwandlungsmethoden  ***/

   public String toString ( )
   {
      String outstr = "" ;

      for ( resetToHead() ;  ! isAtTail() ;  increment() )
         outstr += currentData().toString() ;

      return outstr ;

   }  // end toString()


/***  main-Methode  **/

   public static void main ( String args[] )
   {
      DoubleList myList = new DoubleList() ;

      for ( int i = 0 ;  i < 10 ;  ++i )
         if ( i % 2 == 0 )
	    myList.insertAtHead( " " + i ) ;
         else
	    myList.insertAtTail( " " + i ) ;
      System.out.println( "myList  = " + myList ) ;

      DoubleList secList = new DoubleList( myList ) ;
      secList.remove( " 2" ) ;
      secList.remove( " 5" ) ;
      System.out.println( "secList = " + secList ) ;

   }  // end main()

}  // class DoubleList


