package comaList ;

import java.util.NoSuchElementException ;


/**
 * Klasse fr einfach verkettete Listen
 */
public class LinkedList
{
    /**
     * innere Klasse fr Listenknoten
     */
    private class ListNode                // quasi struct
    {
	public Object   obj ;
	public ListNode next ;

	public ListNode ( )
	{
	    obj  = null ;
	    next = null ;
	}

	public ListNode ( Object myObj )
	{
	    obj  = myObj ;
	    next = null ;
	}
    }  // class ListNode


/***  Daten  ***/

    private ListNode _head ;              // Referenz auf ersten Knoten
    private ListNode _curr ;              // Referenz auf akt. Knoten


/***  Konstruktoren  ***/

    /**
     * default-Konstruktor
     */
    public LinkedList ( )
    {
	_head = _curr = null ;
    }


    /**
     * Kopierkonstruktor
     *
     * @param myList die zu kopierende Liste
     */
    public LinkedList ( LinkedList myList )
    {
	this() ;                                           // ruft LinkedList()

	ListNode tail = null ;

	for ( myList.reset() ;  ! myList.isAtEnd() ;  myList.increment() )
        {
	    ListNode newNode = new ListNode( myList.currentData() ) ;
 
	    if ( tail == null )
		_head = tail      = newNode ;
	    else
		tail  = tail.next = newNode ;
	}
    }  // end LinkedList()


/***  get-Methoden  ***/

    /**
     * @return ist Liste leer?
     */
    public boolean isEmpty ( )
    {
	return ( _head == null ) ;
    }


    /**
     * @return steht akt. Knoten Knoten am Listenende?
     */
    public boolean isAtEnd ( )
    {
	return ( _curr == null ) ;
    }


    /**
     * @return Objekt des akt. Knotens
     */
    public Object currentData ( )
	throws NoSuchElementException
    {
	if ( isAtEnd() )
	    throw new NoSuchElementException( "No current list node!" ) ;

	return _curr.obj ;
    }


    /**
     * @return die aktuelle Anzahl Elemente in der Liste
     */
    public int length ( )
    {
	int len = 0 ;

	for ( reset() ;  ! isAtEnd() ;  increment() )
	    ++len ;

	return len ;

    }  // end length()

    
/***  set-Methoden  ***/

    /**
     * akt. Knoten auf Anfang setzen
     */
    public void reset ( )
    {
	_curr = _head ;
    }


    /**
     * rckt akt. Knoten ein Element weiter
     */
    public void increment ( )
    {
	if ( ! isAtEnd() )                       // bleibt sonst am Ende stehen
	    _curr = _curr.next ;
    }


    /**
     * fgt neuen Knoten am Anfang der Liste ein
     *
     * @param obj einzufgender Knoten
     */
    public void insert ( Object obj )
    {
	ListNode newNode = new ListNode( obj ) ;    // constructor for ListNode

	newNode.next = _head ;
	_head        = newNode ;
    }

    
    /**
     * nimmt alle Knoten mit geg. Objekt aus der Liste
     *
     * @param obj zu suchendes Objekt
     */
    public void remove ( Object obj )
    {
	ListNode lastNode = null ;

	for ( reset() ;  ! isAtEnd() ;  increment() )
	{
	    if ( obj.equals( _curr.obj ) )
		if ( lastNode == null )
		    _head         = _curr.next ;
		else
		    lastNode.next = _curr.next ;

	    lastNode = _curr ;

	}  // end for ( _curr )

    }  // end remove()


    /**
     * nimmt Knoten _curr aus der Liste
     */
    public void removeCurrent ( )
    {
	ListNode delNode  = _curr ;                // Knoten zum Lschen merken
	ListNode lastNode = null ;

	for ( reset() ;  ! isAtEnd() ;  increment() )
	{
	    if ( _curr == delNode )                    // _curr wieder gefunden
	    {
		if ( lastNode == null )
		    _head         = _curr.next ;
		else
		    lastNode.next = _curr.next ;

		break ;                    // kann nur ein solches Objekt geben
	    }

	    lastNode = _curr ;

	}  // end for ( _curr )

    }  // end remove()


/***  Umwandlungsmethoden  ***/

    /**
     * @return in einen String der Objekte umgewandelte Liste
     */
    public String toString ( )
    {
	StringBuffer strbuf = new StringBuffer() ;

	for ( reset() ;  ! isAtEnd() ;  increment() )
	    strbuf.append( currentData().toString() ) ;

	return strbuf.toString() ;

    }  // end toString()


/***  main-Methode  **/

    /**
     * dient hier zum Test der Methoden
     *
     * @param Kommandozeile
     */
    public static void main ( String args[] )
    {
	LinkedList myList = new LinkedList() ;

	for ( int i = 0 ;  i < 10 ;  ++i )
	    myList.insert( " " + i ) ;
	System.out.println( "myList  = " + myList ) ;

	LinkedList secList = new LinkedList( myList ) ;
	secList.remove( " 5" ) ;
	System.out.println( "secList = " + secList ) ;

    }  // end main()

}  // class LinkedList
